/**
 * @author Elliott Sprehn (esprehn@gmail.com)
 */
describe('PostController', function(){
  var controller, root, browser;

  beforeEach(function(){
    root = angular.scope();
    browser = root.$service('$browser');
    controller = root.$new(PostController);

    this.addMatchers({
      /**
       * Angular adds methods with $ prefix and equals ignores them, so use
       * that since jasmine will think the values are different even though they
       * aren't.
       *
       * @param {Object} value Value to compare.
       * @return {boolean}
       */
      toEqual: function(value) {
        return angular.equals(this.actual, value);
      },

      /**
       * Matchers to compare times with minute precision.
       *
       * @param {number} time Time to compare
       * @return {boolean}
       */
      toEqualMinute: function(time) {
        time = new Date(time);
        var timeTruncated = new Date(time.getFullYear(), time.getMonth(),
            time.getDate(), time.getHours(), time.getMinutes()).getTime();
        return timeTruncated === this.actual;
      }
    });
  });

  it('should load posts all posts on initial page load', function() {
    var posts = [
      {time: new Date().getTime(), text: 'awesome post'}
    ];
    browser.xhr.expectGET('/posts?search=').respond(posts);
    controller.load();
    browser.xhr.flush();
    expect(controller.posts).toEqual(posts);
  });

  it('should filter posts when search field changes', function() {
    var posts = [
      {time: new Date().getTime(), text: 'awesome post'}
    ];
    browser.xhr.expectGET('/posts?search=').respond(posts);
    browser.xhr.expectGET('/posts?search=keywords').respond([]);
    controller.load();
    browser.xhr.flush();
    expect(controller.posts).toEqual(posts);
    controller.search = 'keywords';
    controller.$eval();
    browser.xhr.flush();
    expect(controller.posts).toEqual([]);
  });

  it('should reload posts after new post', function() {
    var posts = [
      {time: new Date().getTime(), text: 'awesome post'}
    ];
    browser.xhr.expectGET('/posts?search=').respond([]);
    browser.xhr.expectPOST('/posts', {text: 'New post'}).
        respond({success: true});
    controller.load();
    browser.xhr.flush();
    browser.xhr.expectGET('/posts?search=').respond(posts);
    controller.text = 'New post';
    controller.post();
    browser.xhr.flush();
    expect(controller.posts).toEqual(posts);
  });

  it('should clear the post field after posting', function() {
    browser.xhr.expectGET('/posts?search=').respond([]);
    browser.xhr.expectPOST('/posts', {text: 'New post'}).
        respond({success: true});
    controller.text = 'New post';
    controller.post();
    browser.xhr.flush();
    expect(controller.text).toBe("");
  });

  it('should show graph of post counts aggregated by minute', function() {
    var time = new Date().getTime();
    browser.xhr.expectGET('/posts?search=').respond([
      {time: time, text: ''},
      {time: time, text: ''},
      {time: time, text: ''},
      {time: time + 1000 * 60 * 5, text: ''}
    ]);
    controller.load();
    browser.xhr.flush();
    var data = controller.getPostsPerMinute();
    expect(data[0].time).toEqualMinute(time);
    expect(data[0].value).toEqual(3);
  });

});
